import time


class Clock:

    def wall_time(self):
        raise NotImplementedError("use a concrete implementation")

    def monotonic_time(self):
        raise NotImplementedError("use a concrete implementation")


class SystemClock(Clock):

    def __init__(self):
        try:
            # perf_counter was added in 3.3, use the higher resolution timer
            # if available, otherwise fallback to the `time.time()`
            time.perf_counter()
            self._monotonic_timer = lambda: time.perf_counter()
        except AttributeError:
            self._monotonic_timer = lambda: time.time()

    def wall_time(self):
        return time.time()

    def monotonic_time(self):
        return self._monotonic_timer()


class ManualClock(Clock):

    def __init__(self, wallInit=0, monotonicInit=0):
        self._wall = wallInit
        self._monotonic = monotonicInit

    def wall_time(self):
        return self._wall

    def monotonic_time(self):
        return self._monotonic

    def set_wall_time(self, t):
        self._wall = t

    def set_monotonic_time(self, t):
        self._monotonic = t



import gzip
import io
import json
import logging
import sys

try:
    import urllib2
except ImportError:
    # python3 renames urllib2 to urllib.request
    import urllib.request as urllib2

logger = logging.getLogger("spectator.HttpClient")


class HttpClient:

    def __init__(self, registry, timeout=1):
        self._registry = registry
        self._timeout = timeout

    def _compress(self, entity):
        out = io.BytesIO()
        with gzip.GzipFile(fileobj=out, mode="w") as f:
            f.write(entity.encode('utf-8'))
        return out.getvalue()

    def _add_status_tags(self, tags, code):
        tags["statusCode"] = "{}".format(code)
        tags["status"] = "{}xx".format(int(code / 100))

    def _read_response(self, response):
        if response.info().get('Content-Encoding') == 'gzip':
            buf = io.BytesIO(response.read())
            f = gzip.GzipFile(fileobj=buf)
            return f.read()
        else:
            return response.read()

    def _read_error(self, response):
        if response.headers.get('Content-Encoding', 'identity') == 'gzip':
            buf = io.BytesIO(response.read())
            f = gzip.GzipFile(fileobj=buf)
            return f.read()
        else:
            return response.read()

    def post_json(self, uri, data):
        headers = {
            "Accept-Encoding": "gzip",
            "Content-Encoding": "gzip",
            "Content-Type": "application/json"
        }

        tags = {
            "client": "spectator-py",
            "method": "POST",
            "mode": "http-client"
        }

        if type(data) is str:
            entity = data
        else:
            entity = json.dumps(data)

        logger.debug("posting data to %s, payload: %s", uri, entity)
        request = urllib2.Request(uri, self._compress(entity), headers)

        start = self._registry.clock().monotonic_time()
        try:
            response = urllib2.urlopen(request, timeout=self._timeout)
            self._add_status_tags(tags, response.code)
            msg = self._read_response(response)
            logger.debug("request succeeded (%d): %s", response.code, msg)
        except urllib2.HTTPError as e:
            self._add_status_tags(tags, e.code)
            msg = self._read_error(e)
            logger.warning("request failed (%d): %s", e.code, msg)
        except urllib2.URLError as e:
            error = type(e).__name__
            tags["status"] = error
            tags["statusCode"] = error
            logger.warning("request failed: %s", e)
        except:
            e = sys.exc_info()[0]
            error = type(e).__name__
            tags["status"] = error
            tags["statusCode"] = error
            logger.warning("request failed: %s", e)

        duration = self._registry.clock().monotonic_time() - start
        self._registry.timer("http.req.complete", tags).record(duration)



#!/usr/bin/env python

import os
from setuptools import setup


# Utility function to read the README file.
# https://pythonhosted.org/an_example_pypi_project/setuptools.html#setting-up-setup-py
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='netflix-spectator-py',
    version='0.1.11',
    description='Python library for reporting metrics to Atlas.',
    long_description=read('README.md'),
    author='Brian Harrington',
    author_email='netflix-atlas@googlegroups.com',
    license='Apache 2.0',
    url='https://github.com/netflix/spectator-py/',
    packages=['spectator', 'spectator.histogram'],
    install_requires=['future'],
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development',

        # Pick your license as you wish (should match "license" above)
         'License :: OSI Approved :: Apache Software License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ]
)
